package org.nla.cobol;

import java.util.*;


public class CobolCopyBook extends CobolVariableGroup 
{	
	
	/////////////////////////////////////////////////////////////////
	///		Static members
	/////////////////////////////////////////////////////////////////	
	
	/** count of fillers in the copybook */
	protected static int intFillerCount = 1;
	
	/////////////////////////////////////////////////////////////////
	///		Members
	/////////////////////////////////////////////////////////////////	

	/** final map of child elements */
	protected Map<String, CobolElement> finalChildrenAsMap;
	
	/** final list of child elements */
	protected List<CobolElement> finalChildrenAsList;
	
	

	/////////////////////////////////////////////////////////////////
	///		Members
	/////////////////////////////////////////////////////////////////
	
	/**
	 * Default and only constructor
	 */
	public CobolCopyBook() {
		super.setLevelAsString("0");
		setName("ROOT");
		setIndices(new ArrayList<Integer>());
		_lstIndices = new ArrayList<Integer>();
		finalChildrenAsMap = new LinkedHashMap<String, CobolElement>();
		finalChildrenAsList = new ArrayList<CobolElement>();
		ourParentToUsAll = this;
	}


	/////////////////////////////////////////////////////////////////
	///		Methods
	/////////////////////////////////////////////////////////////////
	
	/**
	 * This method builds a tree representation of the elements in the copybook. The elements passed to this method
	 * must be independant element, directly read from the copybook stream. They musn't have any link between each other, 
	 * and the occursed elements must be flagged as such, but not actually occursed.<br> 
	 * This method takes the three following steps to build the tree : <br>
	 * &nbsp;&nbsp; - first, build parent-children relationships between elements (call to buildGroups() )
	 * &nbsp;&nbsp; - second, replicate all the occursed elements (call to buildOccurs() )
	 * &nbsp;&nbsp; - finally, order all the elements and identifiy their positions in the copybook (call to computePositions() )  
	 * @param rawElements	raw cobol elements
	 */
	public void buildCopyBookFromRawElements( ArrayList<CobolElement> rawElements) {
		buildGroups(this, rawElements.listIterator());
		buildOccurs();
		computePositions(1);
	}

	/**
	 * Returns an iterator on all elements names contained in the final map 
	 * @return	an iterator on all elements names contained in the final map
	 */
	protected Iterator<String> keyIterator() {
		return finalChildrenAsMap.keySet().iterator();
	}

	/**
	 * Looks for the element identified by elementName in final map. Returns it if found, null otherwise.
	 * @param 	elementName			name of the element to find 
	 * @return	the actual element if found, null otherwise
	 */
	public CobolElement getCobolElement( String elementName ) {
		return finalChildrenAsMap.get( elementName );
	}
	
	/**
	 * Returns the final list of cobol elements 
	 * @return	the final list of cobol elements
	 */
	public List<CobolElement> getListOfCobolElements() {
		return finalChildrenAsList;
	}
	/**
	 * Returns the number of elements in this group and its children
	 * @return	 Returns the number of elements in this group
	 */
	
	@Override
	public int getCount()	{

		return ( finalChildrenAsList.size() );
	}
}