
////////////////////////////////////////////////////////////////////
// Package
////////////////////////////////////////////////////////////////////

package org.nla.utils;

////////////////////////////////////////////////////////////////////
// Imports
////////////////////////////////////////////////////////////////////

import java.util.Date;
import java.text.SimpleDateFormat;
import java.io.File;


////////////////////////////////////////////////////////////////////
// Class definition
////////////////////////////////////////////////////////////////////

public final class Util
{
   ////////////////////////////////////////////////////////////////////
   // Constants
   ////////////////////////////////////////////////////////////////////

   /** standard timestamp formater */
   private static final SimpleDateFormat STANDARD_FORMAT = new SimpleDateFormat ("dd/MM/yyyy HH:mm:ss");


   ////////////////////////////////////////////////////////////////////
   // Constructors
   ////////////////////////////////////////////////////////////////////
   /**
    * Not instantiable
    */
   private Util()
   {
      // does nothing
   }

   ////////////////////////////////////////////////////////////////////
   // Static methods
   ////////////////////////////////////////////////////////////////////

   /**
    * Returns String representing current timestamp with dd/mm/yyyy hh:mm:ss format
    * @return	String representing current timestamp with dd/mm/yyyy hh:mm:ss format
    */
   public static String now()
   {
      return ( STANDARD_FORMAT.format( new Date() ) );
   }

   
   /**
    * Returns String representing current timestamp with specified format
    * @param strFormat	String representing the format with which to output the current timestamp 
    * @see SimpleDateFormat
    */
   public static String now( String strFormat )
   {
      return ( new SimpleDateFormat( strFormat ).format( new Date() ) );
   }

   
   /**
    * Returns calling method signature in stack trace
    */
   public static String signature()
   {
      return ( signature( 1 ) );
   }

   
   /**
    * Returns nth calling method signature in stack trace
    * @param intStackLevel 	Integer describing the stack level
    */
   public static String signature( int intStackLevel )
   {
      StringBuffer toReturn = new StringBuffer();
      StackTraceElement element = new Throwable().getStackTrace()[intStackLevel];
      toReturn.append( element.getClassName() );
      toReturn.append( "::" );
      toReturn.append( element.getMethodName() );
      return ( toReturn.toString() );
   }

   /**
    * Creates String containing tokenCount times the specified token  
    * @param tokenCount		Number of times the token is to be repeated in returned String
    * @param token			Token to repeat in final String
    */
   public static String buildStringOfNTokens( int tokenCount, String token ) {
	   StringBuilder builder = new StringBuilder();
	   
	   for ( int i = 0; i < tokenCount; ++i ) {
		   builder.append( token );
	   }
	   
	   return builder.toString();
   }
   
   /**
    * Creates all file parent directories if they don't exists
    * @param strPath 	String representing path File or Directory whose parent directories have to be created
    */
   public static void createParentDirs( String strPath )
   {
	   createParentDirs( new File( strPath ) );
   }


   /**
    * Creates all file parent directories if they don't exists
    * @param clsFile 	File or Directory whose parent directories have to be created
    */
   public static void createParentDirs( File clsFile )
   {
      if ( clsFile.isDirectory() )
      {
    	  clsFile.mkdirs();
      }
      else // if ( clsFile.isFile() )
      {
    	  clsFile.getParentFile().mkdirs();
      }
   }
   



   ////////////////////////////////////////////////////////////////////
   // Unit testing
   ////////////////////////////////////////////////////////////////////

   public static void main(String[] args)
   {
	   // @test createParentDirs( File )
	   File f = new File ( "e:\\testu\\nico\\biloute\\test_unitaire.cvs" );
	   Util.createParentDirs( f );

	   // @test createParentDirs( String )
	   Util.createParentDirs( "e:\\testu\\nico\\biloute\\test_unitaire_2.cvs" );
	   
	   // @test signature()
	   System.out.println( signature() );
	   
	   // @test signature( int )
	   System.out.println( staticMethod1() );
	   
	   // @test now()
	   System.out.println( now() );
	   
	   // @test now( String )
	   System.out.println( now( "dd!MM" ) );
	   
	   
   }
   
   private static String staticMethod1()
   {
	   return( staticMethod2() );
   }
   
   private static String staticMethod2()
   {
	   return( staticMethod3() );
   }
   
   private static String staticMethod3()
   {
	   return( signature(2) );
   }
}